#!/usr/bin/python2.7

import sys
import argparse
import traceback
import sandglass

def parse_args(args=None):
	parser = argparse.ArgumentParser(prog="sglserver", description="Sandglass Server CLI tool")
	subparsers = parser.add_subparsers(help="available commands")
	parser.add_argument("--debug", "-D", help="enable debugging", dest="debug", action="store_true")

	# init command

	parser_init = subparsers.add_parser("init", help="Initialize server-side repository")
	parser_init.set_defaults(command=init)

	parser_init.add_argument("--force", action="store_true", help="Delete any existning content.")

	# status command

	parser_status = subparsers.add_parser("status", help="Show list of synchronized clients.")
	parser_status.set_defaults(command=status)

	# log command

	parser_log = subparsers.add_parser("log", help="Show client history")
	parser_log.set_defaults(command=log)

	parser_log.add_argument("client", help="Client name")
	parser_log.add_argument("files", metavar="file", nargs="*", help="File, directory, or a @package.")

	# delete command

	parser_delete = subparsers.add_parser("delete", help="Delete client history")
	parser_delete.set_defaults(command=delete)

	parser_delete.add_argument("client", help="Client name.")

	# show command

	parse_show = subparsers.add_parser("show", help="Show revision content. If client name is supplied instead "
			"of revision name, last revision from client will be used.")
	parse_show.set_defaults(command=show)

	parse_show.add_argument("commit", help="Revision or client name.")
	parse_show.add_argument("--files", metavar="file", nargs="*", help="Limit the output to the given filenames.")

	# diff command

	parse_diff = subparsers.add_parser("diff", help="Show difference between two revisions. If client "
			"name is supplied instead of revision name, last revision from client will be used.")
	parse_diff.set_defaults(command=diff)

	parse_diff.add_argument("commit_old", help="Revision or client name.")
	parse_diff.add_argument("commit_new", help="Revision or client name.")
	parse_diff.add_argument("--files", metavar="file", nargs="*", help="Limit the output to the given "
			"filenames")

	# pick command

	parse_pick = subparsers.add_parser("pick", help="Apply the changes introduced in one commit to other system.")
	parse_pick.set_defaults(command=pick)

	parse_pick.add_argument("commit", help="Commit name to by applied.")
	parse_pick.add_argument("client", help="Client name the commit will be applied on.")

	return parser.parse_args(args)

def init(args, sglserver):
	sglserver.init(args.force)
	print "Initialized Sandglass Server repository."

def status(args, sglserver):
	clients = sglserver.status()
	print "# List of synchronized clients"
	print "# client        last rev.  date"
	for client in sorted(clients):
		(revision, date) = clients[client]
		print "%-15s %-10s %s" % (client, revision, date)

def log(args, sglserver):
	print sglserver.log(args.client, args.files)

def delete(args, sglserver):
	sglserver.delete(args.client)
	print "History of client '%s' sucessfully removed." % args.client

def show(args, sglserver):
	output = sglserver.show(args.commit, args.files)
	if output:
		print output

def diff(args, sglserver):
	output = sglserver.diff(args.commit_old, args.commit_new, files=args.files)
	if output:
		print output

def pick(args, sglserver):
	sglserver.pick(args.commit, args.client)
	print "Changes in '%s' sucessfully applied on client '%s'." % (args.commit, args.client)

if __name__ == "__main__":
	args = parse_args()
	try:
		sglserver = sandglass.SandglassServer()
		args.command(args, sglserver)
	except IOError as e:
		if e.errno == errno.EPIPE:
			pass
		else:
			print >> sys.stderr, "ERROR: %s" % e
	except Exception as e:
		print >> sys.stderr, "ERROR: %s" % e
		if args.debug:
			traceback.print_exc(sys.stderr)
		sys.exit(1)

	sys.exit(0)
