#!/usr/bin/python2.7

import unittest
from mocker import Mocker
import os
import stat

from setup import *
from sandglass.fileinfo import attributes
import sandglass

class FileInfoTest(TestCaseTempDir):
	"""
	Module sandglass.fileinfo Unit Test.
	"""

	def test_attribute(self):
		"""Attribute 'save' and 'load' methods."""

		filename = os.path.join(self.temp_dir, "foo")
		open(filename, "w").write("foo")
		os.chmod(filename, 0641)

		obj = attributes.Attribute(filename, package = "foo")
		obj.load()

		self.assertTrue(obj.filename, filename)
		self.assertTrue(obj.package, "foo")
		self.assertTrue(obj.mode, 0641)

		os.chmod(filename, 0600)
		obj.save()

		status = os.lstat(filename)
		self.assertTrue(stat.S_IMODE(status.st_mode), 0641)

	def test_to_string(self):
		"""Method 'to_string' returns expected string."""

		filename = os.path.join(self.temp_dir, "foo")
		open(filename, "w").write("foo")
		os.chmod(filename, 0641)

		obj = attributes.Attribute(filename, package = "foo")
		obj.load()

		parts = obj.to_string().split()
		self.assertEquals(len(parts), 6)
		self.assertEquals(parts[0], filename)
		self.assertEquals(parts[1], "foo")
		self.assertEquals(parts[2], "417")

	def test_diff_headers(self):
		"""Test if diff headers are generated."""

		filename_foo = os.path.join(self.temp_dir, "foo")
		open(filename_foo, "w").write("foo")
		filename_bar = os.path.join(self.temp_dir, "bar")
		open(filename_bar, "w").write("bar")

		obj_foo = attributes.Attribute(filename_foo, package="foo")
		obj_bar = attributes.Attribute(filename_bar, package="bar")

		headers = attributes.Attribute.gen_diff_headers(obj_foo, obj_bar)
		self.assertTrue(headers[0].startswith("mode"))
		self.assertTrue(headers[1].startswith("owner"))
		self.assertTrue(headers[2].startswith("context"))

if __name__ == "__main__":
	unittest.main()
